//%define debug
%include "asm.inc"
%include "rsrc.inc"

global Application_LoadSettings
global Application_SaveSettings
extern $wszPropLog ; LogDlg.asm
extern $wszInitialClassProp
extern AllocHeap
extern FreeHeap

HKEY_CURRENT_USER  equ 0x80000001
REG_OPTION_NON_VOLATILE equ 0
ERROR_MORE_DATA    equ 234
REG_BINARY         equ 3
KEY_WRITE          equ 0x20006
KEY_READ           equ 0x20019
KEY_ALL_ACCESS     equ 0xF003F
LVM_GETHEADER      equ 0x1000 + 31
HDM_GETITEMCOUNT   equ 0x1200
LVM_GETCOLUMNWIDTH equ 0x1000 + 29
LVM_GETCOLUMNORDERARRAY equ 0x1000 + 59
LVM_SETCOLUMNORDERARRAY equ 0x1000 + 58
LVM_SETCOLUMNWIDTH      equ 0x1000 + 30
CB_GETLBTEXT equ 0x148
CB_GETCURSEL equ 0x147
CB_GETLBTEXTLEN equ 0x149

struc WINDOWPLACEMENT
.length           resd 1
.flags            resd 1
.showCmd          resd 1
.ptMinPosition    resd 2
.ptMaxPosition    resd 2
.rcNormalPosition resd 4
endstruc

struc MYSETINGS
.wpl_main       resb WINDOWPLACEMENT_size
.wpl_log        resb WINDOWPLACEMENT_size
.i_hash         resd 1 // simple checksum; not done
.w_comboW       resw 1 // combo width
.b_ShowAllProps resb 1 // show unassigned properties
.b_ncol         resb 1 // number of columns
	///// listview columns //////
.i_order        resd 0 ; 0 - columns<<2
.i_width        resd 0 ; 0 - columns<<2
.s_lastclass    resw 1 ; unknown size; min 2; unicode
endstruc

extern isMenuItemChecked
extern LogDlgAddString
extern SystemError
extern $wszComboWProp


PROC Application_LoadSettings, 12, hwnd
	LOCAL  hKey
	LOCAL  datalen
	LOCAL  nColumns

	call   AllocHeap, MYSETINGS_size
	mov    esi,eax
	and    eax,eax
	jz     near .quit
	mov    dword hKey,0
	invoke RegOpenKeyExW, HKEY_CURRENT_USER, $wszRegPath, 0, KEY_READ, offset hKey

	and    eax,eax
	jz    .opened
	call   SystemError
	push   eax
	call   LogDlgAddString, hwnd, eax
	invoke LocalFree
	jmp    near .free
.opened
	mov    dword datalen, 0
	invoke RegQueryValueExW, hKey, $wszSettings, 0, 0, esi, offset datalen
	cmp    eax, ERROR_MORE_DATA
	jne    near .close
	call   FreeHeap, esi
	call   AllocHeap, datalen
	mov    esi,eax

	invoke RegQueryValueExW, hKey, $wszSettings, 0, 0, esi, offset datalen

	movzx  eax,byte[esi+MYSETINGS.b_ncol]
	and    eax,eax
	mov    nColumns,eax
	jz     near .nocolumns

	// set order array
	lea    edi, [esi+MYSETINGS.i_order]
	invoke SendDlgItemMessageW, hwnd, IDC_LSV1, LVM_SETCOLUMNORDERARRAY, nColumns, edi
	// set columns widths
	xor    ebx,ebx
	mov    eax,nColumns
	shl    eax,2
	add    edi,eax

.nxtcol:
	invoke SendDlgItemMessageW, hwnd, IDC_LSV1, LVM_SETCOLUMNWIDTH, ebx, [edi]
	add    edi,4
	inc    ebx
	cmp    ebx,nColumns
	jb    .nxtcol

.nocolumns: // EDI points to MYSETINGS.s_lastclass

	invoke SetWindowPlacement, hwnd, offset [esi+MYSETINGS.wpl_main]
	invoke GetPropW, hwnd, $wszPropLog
	mov    edx,eax
	invoke SetWindowPlacement, edx, offset [esi+MYSETINGS.wpl_log]
	///////////////
	movzx  eax,byte[esi+MYSETINGS.b_ShowAllProps]
	and    al,1
	shl    al,3
	push   eax
	invoke GetMenu, hwnd
	invoke CheckMenuItem, eax, IDM_SHOWALLPROPS // [esp]

	movzx  eax,word [esi+MYSETINGS.w_comboW]
	invoke SetPropW, hwnd, $wszComboWProp, eax

	// duplicate MYSETINGS.s_lastclass and save pointer in property

	invoke lstrlenW, edi
	and    eax,eax
	jz    .close
	inc    eax
	add    eax,eax
	invoke AllocHeap, eax
	and    eax,eax
	jz    .close
	mov    ebx,eax
	invoke SetPropW, hwnd, $wszInitialClassProp, ebx
	invoke lstrcpyW, ebx, edi

.close:
	invoke RegCloseKey, hKey
.free:
	call   FreeHeap, esi
.quit:
	%undef hwnd
	%undef hKey
	%undef nColumns
	%undef datalen
	%undef nColumns
ENDPROC

//////////////////////////////////////////////////////////////////////////////////////////

PROC Application_SaveSettings, 20, hwnd
	LOCAL  hHeader
	LOCAL  hKey
	LOCAL  datasize
	LOCAL  nColumns
	LOCAL  comboindex

	xor    eax,eax
	mov    hKey,eax
	mov    nColumns,eax
	dec    eax
	mov    comboindex,eax
	mov    dword datasize, MYSETINGS_size

	// calculate memory size for columns order array + widths
	xor    eax,eax
	invoke SendDlgItemMessageW, hwnd, IDC_LSV1, LVM_GETHEADER, eax,eax
	mov    hHeader,eax
	and    eax,eax
	jz    .nohdr // no header?
	xor    eax,eax
	invoke SendMessageW, hHeader, HDM_GETITEMCOUNT, eax, eax // number of columns
;DebugInt "HDM_GETITEMCOUNT",eax
	inc    eax
	jz    .nohdr
	dec    eax
	mov    nColumns,eax

	shl    eax,3 // count *4 (dword) *2 (order,width)
	add    datasize, eax

.nohdr:
	// get lenght of selected class name
	xor    eax,eax
	invoke SendDlgItemMessageW, hwnd, IDC_CBO1, CB_GETCURSEL, eax, eax
	mov    comboindex,eax
	inc    eax
	jz    .done
	dec    eax
	invoke SendDlgItemMessageW, hwnd, IDC_CBO1, CB_GETLBTEXTLEN, eax, eax
	// The return value is the length of the string, excluding the terminating null character
	add    eax,eax // unicode
	add    datasize, eax

.done:
	// allocate memory for all this data. don't touch ESI !
	call   AllocHeap, datasize
	mov    esi,eax
	and    eax,eax
	jz     near .quit

	// get combo width
	invoke GetPropW, hwnd, $wszComboWProp
	mov    [esi+MYSETINGS.w_comboW],ax

	// save two windows placements: main and log window
	mov    eax, WINDOWPLACEMENT_size
	mov    [esi+MYSETINGS.wpl_main + WINDOWPLACEMENT.length],eax
	mov    [esi+MYSETINGS.wpl_log  + WINDOWPLACEMENT.length],eax
	invoke GetWindowPlacement, hwnd, offset [esi+MYSETINGS.wpl_main]
	invoke GetPropW, hwnd, $wszPropLog
	mov    edx,eax
	invoke GetWindowPlacement, edx, offset [esi+MYSETINGS.wpl_log]

	////////////////////////////
	invoke GetMenu, hwnd
	call   isMenuItemChecked, eax, IDM_SHOWALLPROPS
	mov    [esi+MYSETINGS.b_ShowAllProps],al


	lea    edi, [esi+MYSETINGS.i_order]

	////////////////////////////
	// column order array
	mov    eax,nColumns
	mov    [esi+MYSETINGS.b_ncol],al
	and    eax,eax
	jz    .nocolumns
	invoke SendDlgItemMessageW, hwnd, IDC_LSV1, LVM_GETCOLUMNORDERARRAY, nColumns, edi
	mov    eax,nColumns
	shl    eax,2
	add    edi,eax

	// get column width's
	xor    ebx,ebx
.nxtcol:
	invoke SendDlgItemMessageW, hwnd, IDC_LSV1, LVM_GETCOLUMNWIDTH, ebx, 0
	mov    [edi],eax
	add    edi,4
	inc    ebx
	cmp    ebx,nColumns
	jne   .nxtcol

.nocolumns:

	// get lase selected class name; EDI points to MYSETINGS.s_lastclass
	mov    eax,comboindex
	inc    eax
	jz    .save
	dec    eax
	invoke SendDlgItemMessageW, hwnd, IDC_CBO1, CB_GETLBTEXT, eax, edi

.save:
	// save
	invoke RegCreateKeyExW, HKEY_CURRENT_USER, $wszRegPath, 0, 0, 0, KEY_ALL_ACCESS, 0, offset hKey, offset hHeader
	and    eax,eax
	jnz   .free
	invoke RegSetValueExW, hKey, $wszSettings, 0, REG_BINARY, esi, datasize
	invoke RegCloseKey, hKey
.free
	call   FreeHeap, esi

.quit
	%undef hwnd
ENDPROC




segment .data align=4
align 16
$wszRegPath      db "\LSoftware\sapero\WMI Viewer"
$wszSettings     db "\LSettings"

